package Flock;

import java.awt.*;

/**
 * Clase Basada en tutoriales
 * 
 * 
 */
class Bird
{

    protected static int SeparationRange;
    
  
    protected static int DetectionRange;

    static boolean showRanges = false;
    
    protected Point location = new Point(0,0);
    
    private int currentTheta;
    
    protected Color color;

    protected static Dimension map = new Dimension( 500, 500 );
    
    private double currentSpeed; 
    
    private int maxTurnTheta;

    /**
     * This is the constructor for the birds.
     *
     * @param  x X
     * @param  y Y
     * @param  theta Direccion inicial
     * @param  birdColor Color del Bird
     */
     Bird(int x, int y, int theta, Color birdColor) {
        location.x = x;
        location.y = y;
        currentTheta = theta;
        color = birdColor;
    }

    /**
     * Constructor
     * @param  Que color de Bird se quiere.
     */
    Bird(Color birdColor) {
       
        this((int)(Math.random() * map.width), (int)(Math.random() * map.height), (int)(Math.random() * 360), birdColor);
    }
    
    /**
     * mover el Bird
     *
     * @param  la direccion donde se deberia mover, este valor se obtiene usando un metodo de Flock.
     */
    public void move(int newHeading) {
        int left = (newHeading - currentTheta + 360) % 360;
        int right = (currentTheta - newHeading + 360) % 360;
        
        int thetaChange = 0;
        if (left < right) {
            thetaChange = Math.min(maxTurnTheta, left);
        }
        else {
            thetaChange = -Math.min(maxTurnTheta, right);
        }
        
        currentTheta = (currentTheta + thetaChange + 360) % 360;
        
        location.x += (int)(currentSpeed * Math.cos(currentTheta * Math.PI/180)) + map.width;
        location.x %= map.width;
        location.y -= (int)(currentSpeed * Math.sin(currentTheta * Math.PI/180)) - map.height;
        location.y %= map.height;
    }
    
    /**
     * Dibujar Bird, se usao arc porque es un objeto base que tiene direccion
     *
     * 
     */
    public void draw(Graphics g) {
        g.setColor(this.color);
        g.fillArc(location.x - 12, location.y - 12, 24, 24, currentTheta + 180 - 20, 40);
        
        if (showRanges) {
            drawRanges(g);
        }
    }
    
    /**
     * Bibuja los rangos, Uso de tutorial. Aunque no es tan influyente
     * 
     */
    public void drawRanges(Graphics g) {
        drawCircles(g, location.x, location.y);
        
        int radius = Obstacle.DetectionRange;
        
        boolean top = (location.y < DetectionRange);
        boolean bottom = (location.y > map.height - DetectionRange);
        if (location.x < DetectionRange) { // if left
            drawCircles(g, map.width + location.x, location.y);
            if (top) {
                drawCircles(g, map.width + location.x, map.height + location.y);
            }
            else if (bottom) {
                drawCircles(g, map.width + location.x, location.y - map.height);
            }
        } else if (location.x > map.width - DetectionRange) { // if right
            drawCircles(g, location.x - map.width, location.y);
            if (top) {
                drawCircles(g, location.x - map.width, map.height + location.y);
            }
            else if (bottom) {
                drawCircles(g, location.x - map.width, location.y - map.height);
            }
        }
        if (top) {
            drawCircles(g, location.x, map.height + location.y);
        }
        else if (bottom) {
            drawCircles(g, location.x, location.y - map.height);
        }
    }
    
    /**
     * Metodo usado por drawRange
     * 
     * @param  g El Ghaphic.
     * @param  x posicion X
     * @param  y posicion Y
     */
    protected void drawCircles( Graphics g, int x, int y ) {
        g.setColor(new Color((int)color.getRed()/2, (int)color.getGreen()/2, (int)color.getBlue()/2 ));
        g.drawOval(x-DetectionRange, y-DetectionRange, 2*DetectionRange, 2*DetectionRange);
        g.setColor(color);
        g.drawOval(x-SeparationRange, y-SeparationRange, 2*SeparationRange, 2*SeparationRange);
    }
 
    /**
     * La distancia entre 2 Bird
     *
     * @param  el otro Bird a medir
     * @return la distancia entre 2 Bird
     */
    public int getDistance(Bird otherBird) {
        int dX = otherBird.getLocation().x - location.x;
        int dY = otherBird.getLocation().y - location.y;
        
        return (int)Math.sqrt( Math.pow( dX, 2 ) + Math.pow( dY, 2 ));
    }
    
    /**
     * La distancia entre el Bird y el punto
     *
     * @param p el punto a medir
     * @return distancia entre Bird y el punto
     */
    public int getDistance(Point p) {
        int dX = p.x - location.x;
        int dY = p.y - location.y;
        
        return (int)Math.sqrt( Math.pow( dX, 2 ) + Math.pow( dY, 2 ));
    }
 
    /**
     * 
     *
     * @param  la dimesion donde los Bird se mueven
     */
    static void setMapSize(Dimension newSize) {
        map = newSize;
    }
 
    /**
     * obtiene maxima rotacion
     *
     * @return maxima rotacion.
     */
    public int getMaxTurnTheta() {
        return maxTurnTheta;
    }
    
    /**
     * Nueva maxima rotacion
     * 
     * @param  nueva rotacion
     */
    public void setMaxRotacion(int theta)
    {
        maxTurnTheta = theta;
    }
    
    /**
     * Obten la direccion del Bird
     *
     * @return  la direccion actual donde se mueve el Bird
     */
    public int getTheta() {
        return currentTheta;
    }
 
    /**
     * El bird donde esta
     *
     * @return  La ubicacion del Bird
     */
    public Point getLocation() {
        return location;
    }

    /**
     * Setiar Nueva velocidad del Bird
     *
     * @param  Velocidad del Bird
     */
    public void setSpeed( double speed ) {
        currentSpeed = speed;
    }
    
    /**
     * Obtiene el valor del color de un Bird
     *
     * @return  el Color de un Bird
     */
    public Color getColor() {
        return color;
    }
}